<?php
/**
 * @package   AdminTools
 * @copyright 2010-2021 Akeeba Ltd / Nicholas K. Dionysopoulos
 * @license   GNU General Public License version 3, or later
 */

namespace Akeeba\Component\AdminTools\Administrator\Model;

defined('_JEXEC') or die;

use Akeeba\Component\AdminTools\Administrator\Helper\Storage;
use Joomla\CMS\Application\CMSApplication;
use Joomla\CMS\Factory;
use Joomla\CMS\MVC\Model\BaseDatabaseModel;
use Joomla\String\Inflector;

class MainpasswordModel extends BaseDatabaseModel
{
	public $views = [
		'Adminpassword'              => 'COM_ADMINTOOLS_TITLE_ADMINPASSWORD',
		'Badwords'                   => 'COM_ADMINTOOLS_TITLE_BADWORDS',
		'Cleantempdirectory'         => 'COM_ADMINTOOLS_TITLE_CLEANTEMPDIRECTORY',
		'Databasetools'              => 'COM_ADMINTOOLS_TITLE_DBTOOLS',
		'Emergencyoffline'           => 'COM_ADMINTOOLS_TITLE_EMERGENCYOFFLINE',
		'Fixpermissions'             => 'COM_ADMINTOOLS_TITLE_FIXPERMISSIONS',
		'Configurepermissions'       => 'COM_ADMINTOOLS_TITLE_CONFIGUREPERMISSIONS',
		'Htaccessmaker'              => 'COM_ADMINTOOLS_TITLE_HTACCESSMAKER',
		'Nginxconfmaker'             => 'COM_ADMINTOOLS_TITLE_NGINXCONFMAKER',
		'Webconfigmaker'             => 'COM_ADMINTOOLS_TITLE_WEBCONFIGMAKER',
		'Ipautobanhistories'         => 'COM_ADMINTOOLS_TITLE_IPAUTOBANHISTORIES',
		'Autobannedaddresses'        => 'COM_ADMINTOOLS_TITLE_AUTOBANNEDADDRESSES',
		'Disallowlists'              => 'COM_ADMINTOOLS_TITLE_DISALLOWLISTS',
		'Adminallowlists'            => 'COM_ADMINTOOLS_TITLE_ADMINALLOWLISTS',
		'Mainpassword'               => 'COM_ADMINTOOLS_TITLE_MAINPASSWORD',
		'Quickstart'                 => 'COM_ADMINTOOLS_TITLE_QUICKSTART',
		'Urlredirections'            => 'COM_ADMINTOOLS_TITLE_URLREDIRECTIONS',
		'Scanner'                    => 'COM_ADMINTOOLS_TITLE_SCANNER',
		'Scan'                       => 'COM_ADMINTOOLS_TITLE_SCANS',
		'Scanalerts'                 => 'COM_ADMINTOOLS_TITLE_SCANALERTS_MAINPASSWORD',
		'Seoandlinktools'            => 'COM_ADMINTOOLS_TITLE_SEOANDLINKTOOLS',
		'Webapplicationfirewall'     => 'COM_ADMINTOOLS_TITLE_WAF',
		'Wafdenylists'               => 'COM_ADMINTOOLS_TITLE_WAFDENYLISTS',
		'Configurewaf'               => 'COM_ADMINTOOLS_TITLE_CONFIGUREWAF',
		'Wafexceptions'              => 'COM_ADMINTOOLS_TITLE_WAFEXCEPTIONS',
		'Checktempandlogdirectories' => 'COM_ADMINTOOLS_TITLE_CHECKTEMPANDLOGDIRECTORIES',
		'Schedulinginformation'      => 'COM_ADMINTOOLS_TITLE_SCHEDULINGINFORMATION',
		'Exportimport'               => 'COM_ADMINTOOLS_TITLE_EXPORTIMPORT',
		'Blockedrequestslog'         => 'COM_ADMINTOOLS_TITLE_LOG',
		'Unblockip'                  => 'COM_ADMINTOOLS_TITLE_UNBLOCKIP',
		'Tempsuperusers'             => 'COM_ADMINTOOLS_TITLE_TEMPSUPERUSERS',
	];

	/**
	 * Checks if the user should be granted access to the current view,
	 * based on his Main Password setting.
	 *
	 * @param   string  $view  Optional. The string to check. Leave null to use the current view.
	 *
	 * @return  bool
	 */
	public function accessAllowed(?string $view = null): bool
	{
		/** @var CMSApplication $app */
		$app     = Factory::getApplication();
		$session = $app->getSession();

		// Is this a view protected by the Main Password feature?
		if (empty($view))
		{
			$view = $app->input->getCmd('view', 'Controlpanel');
		}

		$inflector = new Inflector();
		$altView   = $inflector->isPlural($view) ? $inflector->toSingular($view) : $inflector->toPlural($view);

		if (!isset($this->views[$view]) && !isset($this->views[$altView]))
		{
			return true;
		}

		// Do we have a Main Password?
		$params   = Storage::getInstance();
		$mainHash = $params->getValue('mainpassword', $params->getValue('masterpassword', ''));

		if (empty($mainHash))
		{
			return true;
		}

		// Compare the main pw with the one the user entered
		$mainHash = md5($mainHash);
		$userHash = $session->get('admintools.userpwhash', '');

		if ($userHash === $mainHash)
		{
			// The password matches, we are allowed to access everything
			return true;
		}

		// The login is invalid. If the view is locked I'll have to kick the user out.
		$lockedviews_raw = $params->getValue('lockedviews', '');

		if (empty($lockedviews_raw))
		{
			// There are no locked views.
			return true;
		}

		$lockedViews = explode(",", $lockedviews_raw);

		if (empty($lockedViews))
		{
			// This shouldn't happen. There are no locked views.
			return true;
		}

		// Special case: view=Blockedrequestslog, task=browse, format=json (graphs) is always allowed.
		$task   = $app->input->get('task', '');
		$format = $app->input->get('format', 'html');

		if (
			(($view == 'Blockedrequestslog') || ($view == 'Blockedrequestslogs')) &&
			(($format == 'json') || $format == 'raw') &&
			(empty($task) || ($task == 'main') || ($task == 'default'))
		)
		{
			return true;
		}

		// Check if the view is locked.
		if (in_array($view, $lockedViews) || in_array($altView, $lockedViews))
		{
			return false;
		}

		return true;
	}

	/**
	 * Compares the user-supplied password against the main password
	 *
	 * @return  bool  True if the passwords match
	 */
	public function hasValidPassword()
	{
		$params = Storage::getInstance();

		$mainHash = $params->getValue('mainpassword', $params->getValue('masterpassword', ''));

		if (empty($mainHash))
		{
			return true;
		}

		$mainHash = md5($mainHash);
		$userHash = Factory::getApplication()->get('admintools.userpwhash', '');

		return hash_equals($mainHash, $userHash);
	}

	/**
	 * Stores the hash of the user's password in the session
	 *
	 * @param   $passwd  string  The password supplied by the user
	 *
	 * @return  void
	 */
	public function setUserPassword($passwd)
	{
		$userHash = md5($passwd);
		Factory::getApplication()->set('admintools.userpwhash', $userHash);
	}

	/**
	 * Saves the Main Password and the protected views list
	 *
	 * @param   string  $mainPassword    The new main password
	 * @param   array   $protectedViews  A list of the views to protect
	 *
	 * @return  void
	 */
	public function saveSettings($mainPassword, array $protectedViews)
	{
		$params = Storage::getInstance();

		// Add the new main password (also replaces the old "Master password")
		$params->setValue('masterpassword', null);
		$params->setValue('mainpassword', $mainPassword);

		// Add the protected views
		if (!empty($mainPassword) && !in_array('Mainpassword', $protectedViews))
		{
			$protectedViews[] = 'Mainpassword';
		}

		$params->setValue('lockedviews', implode(',', $protectedViews));

		$params->save();
	}

	/**
	 * Get a list of the views which can be locked down and their lockdown status
	 *
	 * @return  array
	 */
	public function &getItemList()
	{
		$lockedViews = [];

		$params = Storage::getInstance();

		$lockedViewsRaw = $params->getValue('lockedviews', '');

		if (!empty($lockedViewsRaw))
		{
			$lockedViews = explode(",", $lockedViewsRaw);
		}

		$views = [];

		foreach ($this->views as $view => $langKey)
		{
			// I will only show a view if it actually exists as a View directory or Controller file.
			$viewDir = JPATH_ADMINISTRATOR . '/components/com_admintools/src/View/' . $view;
			$controllerFile = JPATH_ADMINISTRATOR . '/components/com_admintools/src/Controller/' . $view . 'Controller.php';

			if (!@is_dir($viewDir) && !@is_file($controllerFile))
			{
				continue;
			}

			$views[$view] = [
				in_array($view, $lockedViews),
				$langKey,
			];
		}

		return $views;
	}

	/**
	 * Returns the stored main password
	 *
	 * @return  string
	 */
	public function getMainpassword()
	{
		$params = Storage::getInstance();

		return $params->getValue('mainpassword', $params->getValue('masterpassword', ''));
	}
}